from distutils.core import setup

setup(
  name = "Alfons",
  packages = ["Alfons"],
  version = "0.8.0",
  description = "A library for using the Alfons system",
  author = "Anton Lindroth",
  author_email = "ntoonio@gmail.com",
  url = "https://github.com/ntoonio/Alfons-Python/",
  download_url = "https://github.com/ntoonio/Alfons-Python/archive/0.6.tar.gz",
  keywords = ["home", "automation", "iot"],
  classifiers = [],
)